/************************************************************************
*                                                                       *
*     Copyright Jack Gostl, 1992                                        *
*       CIS ID 76357,2657                                               *
*                                                                       *
*     This program converts a quote and comma delimited text file into  *
*     Windows 3.1 cardfile format.                                      *
*                                                                       *
*     If you find it useful, please drop me a line.                     *
*                                                                       *
************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

void main(int argc, char ** argv)
{
FILE * infile, * outfile;

char line[1024],                    /* input work area                  */
     *fname[2],                     /* file names                       */
     resp,			    /* for Y/N response			*/
     *cptr,*lptr;                   /* pointers for tokenizing input    */

int
	i,                          /* loop counter                     */
        filcnt=0,                   /* file count                       */
        fcnt,                       /* field counter                    */
        idxfld=1,                   /* field to put in index entry      */
        ncount=0;                   /* ncount will become record count  */

long next,                          /* file address for next text item  */
     savetext,                      /* temp address for text            */
     dataptr;                       /* file address for text length     */

char sig[] = "MGC";                 /* cardfile  signature              */
char *crnl = "\r\n";                /* cr-nl                            */

typedef struct    {                  /* cardfile header: 16 bytes 	*/
  char    initials[3];               /* cardfile signature           	*/
  unsigned int cardcnt;              /* record count                	*/
  char fill1[6]; } HEADER;

typedef struct  {                       /* Struct for card index 	*/
unsigned long data_displ;               /* displacement of record data  */
char fill1[1];
char entry[47]; } INDEX;                /* Index record data            */

typedef struct {
	int fill,len;} DATALEN;         /* structure that describes
					   the field that contains text
					   length of the card           */

char * nextfield(char **);

HEADER header;
INDEX index;
DATALEN datalen={0,0};

if (argc>2)
    for (i=1;i<argc;i++)
        if (*argv[i]=='-') idxfld=atoi(argv[i]+1);    /* get index field */
            else fname[filcnt++]=argv[i];             /* save file name  */
if (argc<3 || filcnt<2) {                             /* test parameters */
    fprintf(stderr,"Usage is:\n");
    fprintf(stderr,"\t%s {-index_field_number} text_file card_file\n",argv[0]);
    exit(8);}

if ( (infile=fopen(fname[0],"r")) == NULL) { /* open input file      */
    perror(fname[0]);exit(8);}

                                            /* get record count     */
while (fgets(line,sizeof(line)-1,infile)!=NULL) ncount++;
fprintf(stderr,"%d lines found\n",ncount);

                                            /* Rewind the input file */
rewind(infile);

					    /* Does file exist?	     */
if ( (outfile=fopen(fname[1],"r")) != NULL) {
	fclose(outfile);
	do {
		fprintf(stderr,"Overwrite %s? ",strupr(fname[1]));
		resp=toupper(getchar());
	   } while (resp!='Y' && resp!='N');
	if (resp=='N') {fclose(infile);exit(8);}
					    }
					    /* Create the output file */
if ( (outfile=fopen(fname[1],"wb")) == NULL) {
    perror(fname[0]);exit(8);}
else {                                              /* Build the header */
   strncpy(header.initials,sig,sizeof(sig));        /* sig values   */
   header.cardcnt=ncount;                           /* rec count    */
   memset(header.fill1,0,sizeof(header.fill1));     /* filler       */
   fwrite(&header,sizeof(header),1,outfile);        /* write header */

  next=sizeof(HEADER)+ncount*sizeof(INDEX);         /* first text addr   */

  for (i=0;i<ncount;i++){                           /* write index data  */
    fgets(line,sizeof(line)-1,infile);              /* get line of input */
    lptr=line;					    /* start of line	*/
    cptr=nextfield(&lptr); 	                    /* first field       */
    index.data_displ=next;                          /* build index entry */
    index.fill1[0]=0;
    fcnt=1;                                         /* init field count  */

    fseek(outfile,next+4,SEEK_SET);              /* go to text           */
    dataptr=next;                               /* addr of length in file  */
    datalen.len=0;                              /* initialize length       */
    while (cptr!=NULL) {

	if (fcnt!=idxfld) {                             /* index field??   */
	  datalen.len+=strlen(cptr)+2;                  /* cumulate length */
	  fwrite(cptr,strlen(cptr),1,outfile);          /* write data      */
	  fwrite(crnl,2,1,outfile);                     /* write cr-nl     */
			}
	else {
	  savetext=ftell(outfile);                      /* save address    */
							/* copy first field
							to index      */
	  strncpy(index.entry,cptr,sizeof(index.entry)-1);
							/* write index */
	 fseek(outfile,sizeof(HEADER)+i*sizeof(INDEX),SEEK_SET);
	 fwrite(&index,sizeof(INDEX),1,outfile);
	 fseek(outfile,savetext,SEEK_SET);              /* reset file  */
	    }
	cptr=nextfield(&lptr);                          /* next string     */
	fcnt++;                                         /* next field      */
            }
    next=ftell(outfile);                                /* save next addr  */
    fseek(outfile,dataptr,SEEK_SET);                    /* write datalaen  */
    fwrite(&datalen,sizeof(datalen),1,outfile);         /* datalen         */
			}
    fclose(outfile);                                    /* close output    */
	}
fclose(infile);                                         /* close input     */
}

/*
	This routine returns the next field. The only reason that it
	is needed is that different programs handle the null case
	differently.
*/
char * nextfield(char ** c)
{
static char field[80];
char *fptr;
if (strlen(*c)==0) return(NULL);	/* nothing to do    */
if ( **c=='\"' ) {      		/* test for "xxxx", */
      (*c)++;
      fptr=strchr(*c,'\"');
      *fptr=0;
      strcpy(field,*c);
      *c=fptr+1;
      if (**c!='\n') (*c)++;
      return(field);
			}
if ( **c==',') {              		/* test for ,, */
	(*c)++;
	field[0]=0;
	return(field);
		}
if (**c=='\n') return(NULL);            /* end of line? */
fptr=strchr(*c,',');			/* simple field */
if (fptr==NULL) {
	strcpy(field,*c);
	*c+=strlen(field);
	return(field);
	}
*fptr=0;strcpy(field,*c);*c=fptr+1;
return(field);
}